using System;
using System.Diagnostics;
using System.Threading;
using System.Windows.Forms;
using System.Reflection;
using System.Drawing;

namespace ReflectionIT.Diagnostics
{
	/// <summary>
	/// Directs tracing or debugging output to a TraceForm
	/// </summary>
	/// <remarks>
	/// Copyright  2002 Reflection IT
	/// 
	/// This software is provided 'as-is', without any express or implied warranty.
	/// In no event will the authors be held liable for any damages arising from the
	/// use of this software.
	/// 
	/// Permission is granted to anyone to use this software for any purpose,
	/// including commercial applications, subject to the following restrictions:
	/// 
	/// 1. The origin of this software must not be misrepresented; you must not claim
	/// that you wrote the original software. 
	/// 
	/// 2. No substantial portion of the source code of this library may be redistributed
	/// without the express written permission of the copyright holders, where
	/// "substantial" is defined as enough code to be recognizably from this library. 
	/// </remarks>
	public class NotificationFormTraceListener : TraceListener, IDisposable
	{

		private ApplicationContext m_context;
		private bool m_closed = false;
		private NotificationForm m_form;
		private Thread m_thread;
		private string _title;

		/// <summary>
		/// Create a TraceFormListener with the Application.ProductName as Title
		/// </summary>
		public NotificationFormTraceListener() : this(Application.ProductName) {
		}

		/// <summary>
		/// Create a TraceFormListener with the given Title
		/// </summary>
		/// <param name="title">Title</param>
		public NotificationFormTraceListener(string title)
		{
			this.Title = title;

			m_thread = new Thread(new ThreadStart(this.Start));
			m_thread.Name = "TraceFormThread";
			m_thread.Start();

			// Wait untill the Form exists
			while (m_form == null) {
				Application.DoEvents();
			}
		}

		~NotificationFormTraceListener() {
			this.Dispose(false);
		}

		/// <summary>
		/// Handle the TraceForm Closing event. Hide the Form and Cancel the Close.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Form_Closing(object sender, System.ComponentModel.CancelEventArgs e) {
			if (!m_closed) {
				e.Cancel = true;
				m_form.Hide();
			}
		}

		/// <summary>
		/// Release all resources
		/// </summary>
		/// <param name="disposing"></param>
		protected override void Dispose(bool disposing) {
			this.Close();
			if (disposing) {
				GC.SuppressFinalize(this);
			}
			base.Dispose(disposing);
		}


		/// <summary>
		/// Close the TraceForm
		/// </summary>
		public override void Close() {
			if (!m_closed) {
				m_closed = true;
                if (m_form.InvokeRequired) {
                    m_form.Invoke(new InvokeDelegate(delegate {
                        m_form.Dispose();
                    }));
                } else {
                    m_form.Dispose();
                }

				m_context.ExitThread();
			}
			base.Close();
		}


		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Form_Exit(object sender, System.EventArgs e) {
			this.Close();
		}

		/// <summary>
		/// Start a secondary thread and open the TraceFrom in it 
		/// </summary>
		private void Start() {
			m_form = new NotificationForm(50, Title);
			m_form.Exit += new EventHandler(this.Form_Exit);

            m_context = new ApplicationContext(m_form);

			// Start a Messageloop
			Application.Run(m_context);
		}

		/// <summary>
		/// Get/set the Title of the Trace Form
		/// </summary>
		public string Title {
			get {
				return (this._title);
			}
			set {
				this._title = value;
			}
		}

		/// <summary>
		/// Writes a category name and a message to the Trace Form.
		/// </summary>
		/// <param name="message">A message to write.</param>
		/// <param name="category">A category name used to organize the output.</param>
		public override void Write(string message, string category) {
			if (m_form != null) {
				m_form.Write(message, category);
			} 
		}

		/// <summary>
		/// Writes a message to the Trace Form.
		/// </summary>
		/// <param name="message">A message to write.</param>
		public override void Write(string message) {
			if (m_form != null) {
				m_form.Write(message);
			}
		}

		/// <summary>
		/// Writes a message to the Trace Form followed by a line terminator.
		/// </summary>
		/// <param name="message">A message to write.</param>
		public override void WriteLine(string message, string category) {
			if (m_form != null) {
				m_form.WriteLine(message, category);
			}
		}

		/// <summary>
		/// Writes a category name and a message to the Trace Form followed by a line terminator.
		/// </summary>
		/// <param name="message">A message to write.</param>
		/// <param name="category">A category name used to organize the output.</param>
		public override void WriteLine(string message) {
			if (m_form != null) {
				m_form.WriteLine(message);
			}
		}

        public override void Flush() {
            base.Flush();
        }


	}
}
